@@ -26,7 +26,7 @@
 #include <string.h>
 
 #include "pango-fontmap.h"
-#include "pango-utils.h"
+#include "pango-impl-utils.h"
 #include "pangocairo-private.h"
 #include "pangocairo-win32.h"
 
@@ -212,7 +212,16 @@ compute_glyph_extents (PangoFont        *font,
   logical_rect->width = 0;
   logical_rect->height = (font_extents.ascent + font_extents.descent) * PANGO_SCALE;
 
-  if (glyph != PANGO_GLYPH_EMPTY)
+  if (glyph == PANGO_GLYPH_EMPTY)
+    {
+      /* already initialized above */
+    }
+  else if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
+    {
+      /* space for the hex box */
+      _pango_cairo_get_glyph_extents_missing(cfont, glyph, ink_rect, logical_rect);
+    }
+  else
     {
       cairo_glyph.index = glyph;
       cairo_glyph.x = 0;
@@ -268,28 +277,6 @@ pango_cairo_win32_font_get_glyph_extents (PangoFont        *font,
     *logical_rect = info->logical_rect;
 }
 
-static void
-quantize_position (int *thickness,
-		   int *position)
-{
-  int thickness_pixels = (*thickness + PANGO_SCALE / 2) / PANGO_SCALE;
-  if (thickness_pixels == 0)
-    thickness_pixels = 1;
-  
-  if (thickness_pixels & 1)
-    {
-      int new_center = ((*position - *thickness / 2) & ~(PANGO_SCALE - 1)) + PANGO_SCALE / 2;
-      *position = new_center + (PANGO_SCALE * thickness_pixels) / 2;
-    }
-  else
-    {
-      int new_center = ((*position - *thickness / 2 + PANGO_SCALE / 2) & ~(PANGO_SCALE - 1));
-      *position = new_center + (PANGO_SCALE * thickness_pixels) / 2;
-    }
-
-  *thickness = thickness_pixels * PANGO_SCALE;
-}
-
 static int
 max_glyph_width (PangoLayout *layout)
 {
@@ -348,11 +335,13 @@ create_metrics_for_context (PangoFont    *font,
   metrics->strikethrough_thickness = metrics->underline_thickness;
   metrics->strikethrough_position = height / 4;
 
-  quantize_position (&metrics->underline_thickness, &metrics->underline_position);
-  quantize_position (&metrics->strikethrough_thickness, &metrics->strikethrough_position);
+  pango_quantize_line_geometry (&metrics->underline_thickness,
+				&metrics->underline_position);
+  pango_quantize_line_geometry (&metrics->strikethrough_thickness,
+				&metrics->strikethrough_position);
 
   layout = pango_layout_new (context);
-  font_desc = pango_font_describe (font);
+  font_desc = pango_font_describe_with_absolute_size (font);
   pango_layout_set_font_description (layout, font_desc);
   pango_layout_set_text (layout, sample_str, -1);      
   pango_layout_get_extents (layout, NULL, &extents);
